"""Re-export shim — real implementation lives in dattos_sdk."""
from dattos_sdk.config import (  # noqa: F401
    CONFIG_DIR,
    ConfigError,
    DattosConfig,
    _ensure_migrated,
    _load_global_config,
    _load_profile_config,
    _profile_override,
    _write_profile_config,
    get_active_profile,
    get_config,
    get_profile_config_path,
    list_profiles,
    save_config,
    save_folder,
    set_active_profile,
    set_profile_override,
)

__all__ = [
    "CONFIG_DIR",
    "ConfigError",
    "DattosConfig",
    "_ensure_migrated",
    "_load_global_config",
    "_load_profile_config",
    "_profile_override",
    "_write_profile_config",
    "get_active_profile",
    "get_config",
    "get_profile_config_path",
    "list_profiles",
    "save_config",
    "save_folder",
    "set_active_profile",
    "set_profile_override",
]
